import sys
import os
from os import environ

if sys.version_info[0] == 3:
    from configparser import ConfigParser
else:
    from ConfigParser import ConfigParser


def plugin_enabled(filepath, environment_variable=None, force=False):
    return force or (config_enabled(filepath) and not environment_disabled(environment_variable))


def config_enabled(filepath):
    try:
        parser = ConfigParser()
        parser.read(filepath)
        return parser.getint('main', 'enabled') == 1
    except:
        return False


def environment_disabled(variable):
    return variable is not None and variable in environ and environ[variable] != ''


def combined_profiles_enabled():
    try:
        from rhsm.profile import EnabledRepos
        # flake8: noqa
        return True
    except ImportError:
        return False


def is_root_user():
    return os.getuid() == 0



import sys
sys.path.append('/usr/share/rhsm')
from rhsm.connection import UEPConnection

try:
    from subscription_manager import action_client
except ImportError:
    from subscription_manager import certmgr

try:
    from subscription_manager.identity import ConsumerIdentity
except ImportError:
    from subscription_manager.certlib import ConsumerIdentity

try:
    from subscription_manager.injectioninit import init_dep_injection
    init_dep_injection()
except ImportError:
    pass


def lookup_consumer_id():
    try:
        certificate = ConsumerIdentity.read()
        return certificate.getConsumerId()
    except IOError:
        return None


def get_uep():
    key = ConsumerIdentity.keypath()
    cert = ConsumerIdentity.certpath()
    uep = UEPConnection(key_file=key, cert_file=cert)
    return uep


def get_manager():
    if 'subscription_manager.action_client' in sys.modules:
        mgr = action_client.ActionClient()
    else:
        # for compatability with subscription-manager >= 1.13
        mgr = certmgr.CertManager(uep=get_uep())
    return mgr



import imp

ENABLED_REPOS_CACHE_FILE = '/var/cache/katello-agent/enabled_repos.json'
PACKAGE_CACHE_FILE = '/var/lib/rhsm/packages/packages.json'
REPOSITORY_PATH = '/etc/yum.repos.d/redhat.repo'
ZYPPER_REPOSITORY_PATH = '/etc/rhsm/zypper.repos.d/redhat.repo'

PACKAGE_PROFILE_PLUGIN_CONF = '/etc/yum/pluginconf.d/package_upload.conf'
ENABLED_REPOS_PLUGIN_CONF = '/etc/yum/pluginconf.d/enabled_repos_upload.conf'

DISABLE_PACKAGE_PROFILE_VAR = 'DISABLE_KATELLO_PACKAGE_PROFILE'
DISABLE_ENABLE_REPOS_VAR = 'DISABLE_KATELLO_ENABLED_REPOS'
PROFILE_CACHE_FILE = '/var/lib/rhsm/cache/profile.json'

try:
    imp.find_module('zypp_plugin')
    ZYPPER = True
except ImportError:
    ZYPPER = False

try:
    imp.find_module('yum')
    YUM = True
except ImportError:
    YUM = False



from gettext import gettext as _
from logging import getLogger

from katello.agent.pulp.handler import PackageHandler, GroupHandler, ErratumHandler
from katello.agent.pulp.report import DispatchReport, HandlerReport


log = getLogger(__name__)


# Handler Roles
SYSTEM = 0
CONTENT = 1


class HandlerNotFound(Exception):
    """
    Handler not found.
    """

    def __init__(self, **criteria):
        """
        @param criteria: The handler search criteria.
        @type criteria: dict
        """
        Exception.__init__(self, criteria)

    def __str__(self):
        return _('No handler for: {c}'.format(self.args[0]))


class Dispatcher(object):
    """
    The dispatch delegates operations to handlers based on
    the type_id specified in the request.
    """

    # Handler mapping.
    handler = {}

    @staticmethod
    def collated(units):
        """
        Content units collated by type_id.

        Args:
            units (collections.Iterable): A list of content units.

        Returns:
            dict: Units collated by type_id.
        """
        collated = {}
        for unit in units:
            type_id = unit['type_id']
            _list = collated.setdefault(type_id, [])
            _list.append(unit['unit_key'])
        return collated

    def install(self, units, options):
        """
        Install content.

        Args:
            units (collections.Iterable): Content to be installed.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        dispatch_report = DispatchReport()
        collated = self.collated(units)
        for type_id, units in collated.items():
            try:
                handler = self.find_handler(type_id, CONTENT)
                _report = handler.install(units, dict(options))
                _report.aggregation_key = type_id
                _report.update(dispatch_report)
            except Exception:
                log.exception(_('Handler failed.'))
                _report = HandlerReport()
                _report.aggregation_key = type_id
                _report.set_failed(HandlerReport.last_exception())
                _report.update(dispatch_report)
        return dispatch_report

    def update(self, units, options):
        """
        Update content.

        Args:
            units (collections.Iterable): Content to be updated.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        dispatch_report = DispatchReport()
        collated = self.collated(units)
        for type_id, units in collated.items():
            try:
                handler = self.find_handler(type_id, CONTENT)
                _report = handler.update(units, dict(options))
                _report.aggregation_key = type_id
                _report.update(dispatch_report)
            except Exception:
                log.exception(_('Handler failed.'))
                _report = HandlerReport()
                _report.aggregation_key = type_id
                _report.set_failed(HandlerReport.last_exception())
                _report.update(dispatch_report)
        return dispatch_report

    def uninstall(self, units, options):
        """
        Uninstall content.

        Args:
            units (collections.Iterable): Content to be uninstalled.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        dispatch_report = DispatchReport()
        collated = self.collated(units)
        for type_id, units in collated.items():
            try:
                handler = self.find_handler(type_id, CONTENT)
                _report = handler.uninstall(units, dict(options))
                _report.aggregation_key = type_id
                _report.update(dispatch_report)
            except Exception:
                log.exception(_('Handler failed.'))
                _report = HandlerReport()
                _report.aggregation_key = type_id
                _report.set_failed(HandlerReport.last_exception())
                _report.update(dispatch_report)
        return dispatch_report

    def find_handler(self, type_id, role):
        """
        Find a handler by type ID and role.

        Args:
            type_id (int): A content type ID.
            role (int): The handler role requested.

        Returns:
            Handler: The requested handler.

        Raises:
            HandlerNotFound: When not found.
        """
        try:
            handler = self.handler[role][type_id]
        except KeyError:
            raise HandlerNotFound(type=type_id)
        else:
            return handler()


# Handler Registration.
Dispatcher.handler = {
    CONTENT: {
        'rpm': PackageHandler,
        'package_group': GroupHandler,
        'erratum': ErratumHandler,
    }
}



from __future__ import absolute_import
from os import path
import subprocess
import sys
try:
    from shutil import which
except ImportError:  # on Python 2
    from distutils.spawn import find_executable as which

# The file is created by /etc/kernel/postinst.d/unattended-upgrades (part of unattended-upgrades pkg)
REBOOT_NEEDED_FLAG = "/var/run/reboot-required"


class AptTracerApp:
    def __init__(self, name, helper, type):
        self.name = name
        self.helper = helper
        self.type = type


def has_needrestart():
    return which("needrestart") is not None


def use_flag():
    apps = []
    if path.isfile(REBOOT_NEEDED_FLAG):
        app = AptTracerApp("kernel", "You will have to reboot your computer", "static")
        apps.append(app)
    return apps


def needrestart():
    try:
        output = subprocess.check_output(["needrestart", "-b"], universal_newlines=True)
        return output.split("\n")
    except OSError:
        raise SystemExit("Please install needrestart")


def use_needrestart():
    apps = []
    services = []
    ksta = None
    output = needrestart()
    for item in output:
        output = item.split(":")
        if output[0] == "NEEDRESTART-KSTA":
            ksta = int(output[1])
        elif output[0] == "NEEDRESTART-SVC":
            services.append(output[1].replace(".service", ""))

    if ksta in (2, 3):
        app = AptTracerApp("kernel", "Please restart your system", "static")
        apps.append(app)
    
    
    for service in services:
        app = AptTracerApp(service, "systemctl restart" + service, "systemd")
        apps.append(app)

    return apps


def collect_apps(plugin=None):
    if has_needrestart:
        apps = use_needrestart()
    else:
        apps = use_flag()
    return apps



from __future__ import absolute_import


from logging import getLogger

from katello.agent.pulp.report import ContentReport

try:
    from .libdnf import Package, PackageGroup, Pattern
except ImportError:
    from .libyum import Package, PackageGroup, Pattern


log = getLogger(__name__)


class ContentHandler(object):
    """
    <Abstract> Content handler.
    Defines the interface for handlers designed to implement CONTENT management requests.
    """

    def install(self, units, options):
        """
        Install content.

        Args:
            units (collections.Iterable): Content to be installed.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        raise NotImplementedError()

    def update(self, units, options):
        """
        Update content.

        Args:
            units (collections.Iterable): Content to be updated.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        raise NotImplementedError()

    def uninstall(self, units, options):
        """
        Uninstall content.

        Args:
            units (collections.Iterable): Content to be uninstalled.
            options (dict): Handler specific options.

        Returns:
            DispatchReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        raise NotImplementedError()


class PackageReport(ContentReport):
    """
    Package (install|update|uninstall) report.
    Calculates the num_changes.
    """

    def set_succeeded(self, details):
        num_changes = \
            len(details['resolved']) + \
            len(details['deps'])
        super(PackageReport, self).set_succeeded(details, num_changes)


class GroupReport(ContentReport):
    """
    Package Group (install|update|uninstall) report.
    Calculates the num_changes.
    """

    def set_succeeded(self, details):
        num_changes = \
            len(details['resolved']) + \
            len(details['deps'])
        super(GroupReport, self).set_succeeded(details, num_changes)


class PackageHandler(ContentHandler):
    """
    The package (rpm) content handler.
    """

    @staticmethod
    def _impl(options):
        commit = options.get('apply', True)
        impl = Package(commit=commit)
        return impl

    def install(self, units, options):
        """
        Install packages.

        Args:
            units (collections.Iterable): Packages to be installed.
            options (dict): Handler specific options.

        Returns:
            PackageReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = PackageReport()
        package = self._impl(options)
        patterns = [Pattern(**u) for u in units]
        details = package.install(patterns)
        if details['failed']:
            report.set_failed(details)
        else:
            report.set_succeeded(details)
        return report

    def update(self, units, options):
        """
        Update packages.

        Args:
            units (collections.Iterable): Packages to be updated.
            options (dict): Handler specific options.

        Returns:
            PackageReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = PackageReport()
        _all = options.get('all', False)
        package = self._impl(options)

        if _all:
            patterns = []
        else:
            patterns = [Pattern(**u) for u in units]

        if patterns or _all:
            details = package.update(patterns)
            if details['failed']:
                report.set_failed(details)
            else:
                report.set_succeeded(details)
        return report

    def uninstall(self, units, options):
        """
        Uninstall packages.

        Args:
            units (collections.Iterable): Packages to be uninstalled.
            options (dict): Handler specific options.

        Returns:
            PackageReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = PackageReport()
        package = self._impl(options)
        patterns = [Pattern(**u) for u in units]
        details = package.uninstall(patterns)
        if details['failed']:
            report.set_failed(details)
        else:
            report.set_succeeded(details)
        return report


class GroupHandler(ContentHandler):
    """
    The package group content handler.
    """

    @staticmethod
    def _impl(options):
        commit = options.get('apply', True)
        impl = PackageGroup(commit=commit)
        return impl

    def install(self, units, options):
        """
        Install package group.

        Args:
            units (collections.Iterable): Groups to be installed.
            options (dict): Handler specific options.

        Returns:
            GroupReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = GroupReport()
        group = self._impl(options)
        names = [key['name'] for key in units]
        details = group.install(names)
        report.set_succeeded(details)
        return report

    def update(self, units, options):
        raise NotImplementedError()

    def uninstall(self, units, options):
        """
        Uninstall packages.

        Args:
            units (collections.Iterable): Groups to be uninstalled.
            options (dict): Handler specific options.

        Returns:
            GroupReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = GroupReport()
        group = self._impl(options)
        names = [key['name'] for key in units]
        details = group.uninstall(names)
        report.set_succeeded(details)
        return report


class ErratumHandler(ContentHandler):
    """
    The package (rpm) content handler.
    """

    @staticmethod
    def _impl(options):
        commit = options.get('apply', True)
        impl = Package(commit=commit)
        return impl

    def install(self, units, options):
        """
        Install Errata.

        Args:
            units (collections.Iterable): Errata to be installed.
            options (dict): Handler specific options.

        Returns:
            PackageReport: A report.

        Notes:
            Unit is: {type_id:<str>, unit_key:<dict>}
        """
        report = PackageReport()
        package = self._impl(options)
        advisories = [unit_key['id'] for unit_key in units]
        details = package.update(advisories=advisories)
        if details['failed']:
            report.set_failed(details)
        else:
            report.set_succeeded(details)
        return report

    def update(self, units, options):
        raise NotImplementedError()

    def uninstall(self, units, options):
        raise NotImplementedError()



import os.path
from katello.utils import ConfigParser
from katello.constants import REPOSITORY_PATH, YUM, ZYPPER

if YUM:
    import yum

class EnabledReport(object):
    def __generate(self):
        if not os.path.exists(self.repofile):
            return {"enabled_repos": {"repos": []}}

        config = ConfigParser()
        config.read(self.repofile)
        enabled_sections = [section for section in config.sections() if config.getboolean(section, "enabled")]
        enabled_repos = [{"repositoryid": section, "baseurl": [self._format_str(config.get(section, "baseurl"))]} for section in enabled_sections]
        return {"enabled_repos": {"repos": enabled_repos}}

    def __init__(self, repo_file=REPOSITORY_PATH):
        """
        :param path: A .repo file path used to filter the report.
        :type path: str
        """
        if YUM:
            self.yb = yum.YumBase()
            self.yb.preconf.debuglevel = 0
        self.repofile = repo_file
        self.content = self.__generate()

    def __str__(self):
        return str(self.content)

    def _format_str(self, repo_url):
        """
        returns a formatted string

        :param repo_url: a repo URL that you want to format
        :type path: str
        """

        if YUM:
            return self._replace_vars(repo_url)
        elif ZYPPER:
            return self._cut_question_mark(repo_url)
        else:
            return repo_url;

    def _cut_question_mark(self, repo_url):
        """
        returns a string where everything after the first occurence of ? is truncated

        :param repo_url: a repo URL that you want to modify
        :type path: str
        """
        return repo_url[:repo_url.find('?')]

    def _replace_vars(self, repo_url):
        """
        returns a string with "$basearch" and "$releasever" replaced.

        :param repo_url: a repo URL that you want to replace $basearch and $releasever in.
        :type path: str
        """

        mappings = {'releasever': self.yb.conf.yumvar['releasever'], 'basearch': self.yb.conf.yumvar['basearch']}

        return repo_url.replace('$releasever', mappings['releasever']).replace('$basearch', mappings['basearch'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU Lesser General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (LGPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of LGPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/lgpl-2.0.txt.
#
# Jeff Ortel <jortel@redhat.com>
#

from setuptools import setup, find_packages

setup(
    name='katello-host-tools',
    version='4.2.0',
    description='Libraries and command-line utilities for keeping Katello clients in sync and up to date',
    author='The Foreman Project',
    author_email='no-email@theforeman.org',
    url='https://github.com/Katello/katello-host-tools',
    license='GPLv2+',
    packages=find_packages(),
    include_package_data=False,
    package_data={ '': [ '../katello/contrib/etc/*/*/*.conf', '../katello/contrib/extra/*' ] },
    classifiers=[
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Operating System :: POSIX',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    entry_points={
        'console_scripts': [
            'katello-enabled-repos-upload=katello.scripts:enabled_repos_upload',
            'katello-package-upload=katello.scripts:package_upload',
            'katello-tracer-upload=katello.scripts:tracer_upload',
        ],
    },
)
